/*****************************************************************************************
 * $Id: orva_rank.sas,v 1.1.2.6.4.1 2006/11/10 18:02:21 stchri Exp $
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  Name: orva_rank.sas                                             
 *  Description: Rank fitting iterations and choose best iteration.
 *
 *	PARAMETERS
 *
 *	measCols    =  List of fitting measures to be ranked with flag for negative measures.
 *                   ex: AD_V 0 LOGL 1 CVM_V 0
 *
 *	parmdata    =  the name of the parameter dataset to rank measures for.
 *
 *	ccUid       =  Current cell. 
 *
 *  selectMeas  =  Measure to use for selection (favorite fit measure).
 *      
 *  sevIter     =  Current Severity Iterations (only valid for frequency).
 * 
 *****************************************************************************************/
%macro orva_rank( measCols      =,
                  parmdata      =,
                  ccUid         =,
                  selectMeas    = LOGL,
                  sevIter       =
                 );

     /*** make sure sorted by CCUID, ITERATION ***/	
     proc sort data=&parmdata; by CCUID ITERATION; run;

     /*** subset by CCUID (and SEVITER) ***/
	 %let ALREADY_SELECTED= ;
     data _temp; set &parmdata; 
       where CCUID=&ccUid and STATUS ne 1
             %if (&sevIter ^= %str()) %then %do; and SEVITER=&sevIter %end;
             ;
	   if SELECTION eq 'S' then call symput("ALREADY_SELECTED", SELECTION);
     run;

     /*** calculate ranks for each measure ****/
     %let nmeas=1;
     %let meas=%scan(&measCols, &nmeas, ' ');
     %let nmeas=2;
     %let negmeas=%scan(&measCols, &nmeas, ' ');
     %do %while (&meas ne );
         proc sort data=_temp;
            by %if &negmeas eq 1 %then %do; descending %end; &meas;
         run;
         data _temp(drop= rank); set _temp;
            length selection $ 1;
            retain rank 1;
            
            /*** skip rank for missing values ***/
            if &meas eq . then do;
               &meas._RANK = .;
            end;
            
            else do;           
                /*** If we on our favorite fit measure, handle selection ***/
	            if upcase("&selectMeas") eq upcase("&meas") and "&ALREADY_SELECTED" eq "" then do;                        
	               if rank eq 1 then selection = 'S';               
	            end;
	            
	            /*** set rank ***/
	            &meas._RANK = rank;
	            rank + 1;
            end;   

         run;
         %let nmeas=%eval(&nmeas+1);
         %let meas=%scan(&measCols, &nmeas, ' ');
         %let nmeas=%eval(&nmeas+1);
         %let negmeas=%scan(&measCols, &nmeas, ' ');
     %end;
     
     /*** merge rows with ranks back into original dataset ***/
     proc sort data=_temp; by CCUID ITERATION; run;
     data &parmdata; merge &parmdata _temp; by CCUID ITERATION; run;
     
     /*** clean up  ***/
	 proc datasets library=work memtype=data nolist nodetails;
	   delete _temp; 
     run; quit;
    
%mend orva_rank;