/*************************************************************************************
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *    NAME: orva_sfitkri_dist.sas                                             
 * PURPOSE: sub-macro of %orva_sfit_kri() called once for each distribution.            
 * SUPPORT: jariou
 *
 * PARAMETERS:
 *    DIST        = Current distribution.
 *    NIDST       = Current distribution number.
 *
 * NOTES: Please see the accompanying doc for %orva_sfit_kri()
 *        The only output of this macro is the dataset _parmnlp_&nidst.
 *
 **************************************************************************************/
%macro orva_sfitkri_dist(dist=,ndist=);

        %let PARAM_COUNT=.;
		%let EXTRA_PARAMS=.;
		%let AVG_P_OBS = -1; /* initializing */


        /* reload macros for this distribution          */
        /* this also gives us the number of parameters  */
        /* in the model (in macro PARAM_COUNT)          */
        %orvd_&dist;

        data _NULL_;
            tmp="&PARAM_COUNT";
            call symput("PARAM_COUNT",trim(left(tmp)));
        run;
        %if %sysevalf(&PARAM_COUNT + &COUNT_KRI > &obsCount,boolean) %then %do;
            %put ERROR: Fewer data points than parameters.;
            %let reason=Fewer data points than parameters;
			%let status = 1;
            %goto LOCAL_EXIT;
        %end;
 
        %let status=0;
        %let reason=Successful;

		%let param_1_init=.;
		%let param_2_init=.;
		%let param_3_init=.;
		%let param_4_init=.;
		%let nparms= 0;
        %InitFit&dist;
        %FitKRI&dist;
        
        %let nobs=0;
	    %let dsid=%sysfunc(open(_cs_,I));

	    %if &dsid ^= 0 %then %do;
	      %let nobs=%sysfunc(attrn(&dsid,nlobs));
	      %let rc=%sysfunc(close(&dsid));
	    %end;

	    %if &nobs eq 0 %then %do;
	      %let status = 1;
	      %let reason = Unknown convergence failure.;
	    %end;
	    %else %do;
	        %if &status=0 %then %do;
	            data _null_;
	                set _cs_;
	                if (substr(reason,1,7) eq "WARNING") then status = 0;
                    if (substr(reason,1,5) eq "ERROR") then status = 1;
	                call symput("status",status);
	                call symput("reason",reason);
	            run;
	        %end;
        %end;
        
        %if &status=0 %then %do;
            /*- have to make the loglikelihood comparable for LNIG and LNG -*/
            %let sumlog= 0;
            %MakeLLGoodKRI&dist;

            data _null_;
                set _logl_;

                if _N_=3 then do;
                call symput("logl",nValue1);
                aic = -2*(nValue1 - &sumlog - &nparms);
                sbc = -2*(nValue1 - &sumlog - &nparms * log(&NExcess)/2);
                call symput("aic",aic);
                call symput("sbc",sbc);
                end;
            run;
            
            %let NLP_COV_COUNT = 0;

            /*  Get the covariance components of the NLP fit    */
            data _NLP_COV_(keep=Param_:);
                set _covmat_  nobs=ObsCount ;
                NLP_COV_COUNT = &PARAM_COUNT * (&PARAM_COUNT + 1)/2;
                call symput("NLP_COV_COUNT",trim(left(NLP_COV_COUNT)));
            run;


            /*  Create the NLP covariance part of the final parameters dataset  */
            data _COV_TMP_(drop=Intercept  &KRIS Param_:);
                set _NLP_COV_;
                array covs[&NLP_COV_COUNT] cov_1-cov_&NLP_COV_COUNT;
                array params[&PARAM_COUNT ] Param_1-Param_%eval(&PARAM_COUNT) ;
                retain cov_:;
                
                do i=1 to _N_;
                    covs(_N_*(_N_-1)/2+i)=params(i);
                end;
                run;

            %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
                %put ERROR: Unable to get rid of extraneous covariance information.;
                %let reason=Unable to get rid of extraneous covariance information.;
                %goto LOCAL_EXIT;
            %end;

            /*  Just keeping the last row so that there is no extra info    */
            data _COV_TMP_;
                set _COV_TMP_ (drop = i  );
                if _N_ ne %eval(&PARAM_COUNT ) then delete;
            run;

            %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
                %put ERROR: Unable to get rid of extraneous covariance information.;
                %let reason=Unable to get rid of extraneous covariance information.;
                %goto LOCAL_EXIT;
            %end;               


            /*  Get estimated parameters from the NLP part  */
            data _ParamEst_(keep= param_: &EXTRA_PARAMS);
                set _Parmnlp_ (where =(index(_type_,"PARMS")>0));
            run;    

            %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
                %put ERROR: Unable to extract the parameter estimates.;
                %let reason=Unable to extract the parameter estimates;
                %goto LOCAL_EXIT;
            %end;

            /*  Format the final returned parameters dataset    */
            data _parmnlp_(drop=Intercept);
                format distribution $12. status 8. reason $100. ;    
                
                set _ParamEst_ ;
                set _linParams_;
                distribution="&dist";
                status=&status;
                reason="&reason";
                logl=&logl-&sumlog;
                aic=&aic;
                sbc=&sbc;
            run;

            %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
                %put ERROR: Unable to prepare dataset for parameter estimates.;
                %let reason=Unable to prepare dataset for parameter estimates;
                %goto LOCAL_EXIT;
            %end;

            /* Decide wether to use the adjusted testing threshold or not */
            %if &AVG_ADJ_GOF_T  NE . %then %do; 
                %ORVA_GFit( data         = _LogWorkdata_KRI_, 
                            var          = KRI_ADJ_AMOUNT, 
                            thresholdVar = KRI_ADJ_Threshold, 
                            ccUid        = &ccUid, 
                            dist         = &dist, 
                            parmdata     = _parmnlp_, 
                            gofThreshold = &AVG_ADJ_GOF_T
                            );  
            %end;
            %else %do;
                %ORVA_GFit( data         = _LogWorkdata_KRI_, 
                            var          = KRI_ADJ_AMOUNT, 
                            thresholdVar = KRI_ADJ_Threshold, 
                            ccUid        = &ccUid, 
                            dist         = &dist, 
                            parmdata     = _parmnlp_, 
                            gofThreshold = 
                            );  
            %end;
                
            %KriFindMoments(parmdata = _parmnlp_, dist = &dist);    
            %NoMissingMOM&dist.(parmdata = _parmnlp_);

			/* Set the macro variables for the parameters of the fitted &dist: param_1, param_2, etc. */
			data _parmtemp_;
				set _parmnlp_(keep=_numeric_);
			run;

			proc transpose data=_parmtemp_ out=_parmtemp_;
			run;

			data _null_;
				set _parmtemp_;
				call symput(_name_, col1);
			run;

			/* calculate the avg_p_obs */
			data _LogWorkdata_KRI_ (drop=p_obs_sum p_obs_count);
				set _LogWorkdata_KRI_ end=last;
				retain p_obs_sum 0 p_obs_count 0;

				if KRI_ADJ_THRESHOLD=0 then do;
					p_obs=1;
				end;
        		else do;
					%Sdf&dist(KRI_ADJ_THRESHOLD, p_obs); /* ????should this be kri_adj_threshold or threshold??? */
				end;
				/* Note: in orva_sfit_kri, kri_adj_threshold is set to threshold / exp(kri_log) */
				/* Note: in orva_freqfit_kri, %Sdf&sdist(THRESHOLD/exp(&KRI_LOG), pScaledExcess);  */
				if ^missing(p_obs) then do;
					p_obs_sum = p_obs_sum + p_obs;
					p_obs_count = p_obs_count + 1;
				end;
				if last then do;
					call symput("avg_p_obs", p_obs_sum / p_obs_count);
				end;
			run;


        %end;
        %else %do;
    %LOCAL_EXIT:
            data _parmnlp_;
                format distribution $12. status 8. reason $100. ;    
                distribution="&dist";
                status=&status;
                reason="&reason";
            run;    
        %end;

        data _parmnlp_&ndist;
            %if &status=0 %then %do;
              merge _parmnlp_  _COV_TMP_;
            %end;
            %else %do;
              set _parmnlp_;
            %end;
            obsCount=&obsCount;
            ParamCount=&PARAM_COUNT;
            COUNT_KRI=&COUNT_KRI;
			avg_p_obs = &AVG_P_OBS;
        run;


        %if &COUNT_KRI > 0 & &status NE 1 %then %do;
            data _parmnlp_&ndist;
                merge _parmnlp_&ndist _KRI_STUFF_;
                array covs[&NLP_COV_COUNT] cov_1-cov_&NLP_COV_COUNT;
                Intercept=&The_Intercept;
                Orig_Param_1 = Param_1;

                %if "&SCALE_TRANSFORM"="LOG" %then %do;
                    Param_1 = Param_1 + Intercept; 
                    Cov_1 = Cov_1 + KCov_0;
                %end;
                %else %do;
                    Param_1 = Param_1 * exp(Intercept); 
                    Cov_1 = exp(2*Intercept)* (Cov_1 + 2*Orig_Param_1);

                    do i=2 to &PARAM_COUNT;
                        Covs[i*(i-1)/2 +1]=exp(Intercept)*Covs[i*(i-1)/2 +1];
                    end;
                %end;
            run;
        %end;

        
%mend orva_sfitkri_dist;

/*- find moments for fitted distribution -*/
%macro KriFindMoments(parmdata=,dist=);
    %orvd_&dist;
    data &parmdata(drop= m1 m2 m3 m4 tmpParam_1);
        set &parmdata;

        tmpParam_1 = Param_1 ;
        %if "&SCALE_TRANSFORM"="LOG" %then %do;
            Param_1 = Param_1 + &KRI_LOG_AVG; 
        %end;
        %else %do;
            Param_1 = Param_1 * exp(&KRI_LOG_AVG); 
        %end;
        %RealMoments&dist;
        Param_1 = tmpParam_1;
        
    run;
%mend KriFindMoments;
