/******************************************************************************
 * $Id: orvd_lng.sas,v 1.1.2.37.4.1 2006/10/19 18:36:13 jariou Exp $
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *   Title: orvd_LNG.sas
 *   
 *   -Provides support for the LogNormal-Gamma distribution-
 *
 *        This file contains macro fragments to implement calculation specific to the 
 *        LogNormal-Gamma distribution
 *
 *  The macro %ORVD_LNG is at the end of this file. It is an empty macro that takes
 *  no parameter and returns no feedback. In normal usage, this macro needs to be 
 *  called before trying to call any other macros depending on the fragments in this file.
 *
 *  Calling the %ORVD_LNG macro makes the SAS system parse and compile the whole file 
 *  so that the other macro fragment then become defined and available for other 
 *  macros to call.
 *
 *  Of course this is only guaranteed to work if there is no syntax error in any of 
 *  the fragments. 
 *
 *  The list of other macro files that rely on fragments in this file:
 *
 *   File                    Fragments it depends on (Approx Line # where fragement is called)
 *   orva_cdfplot.sas        CdfLNG(170)
 *   orva_gfit.sas           CdfLNG(188)
 *   orva_histplot.sas       CdfLNG(176), PdfLNG(177)
 *   orva_los_sim.sas        SdfLNG(263), SimLNG(408)
 *   orva_probplot.sas       CdfLNG(160)
 *   orva_quantplot.sas      QuantLNG(118)
 *   orva_sfitall.sas        FitLNG(304), MakeLLGoodLNG(320), NoMissingMOMLNG(379)
 *   orva_sfitall_model.sas  ModelFitLNG(287), MomentsLNG(391)
 *   orva_sfitall_mom.sas    MomFitLNG(78)
 *
 ******************************************************************************/

%macro LNG_rep;
   do i=1 to &LNG_NQUANT;
        if Param_3 < 0.00002 then do;
            q[i] = sqrt(1 + probit(i/(&LNG_NQUANT+1)) * sqrt(Param_3));
        end;
        else do;        
            q[i]=sqrt(gaminv(i/(&LNG_NQUANT+1),1/param_3)*Param_3);
        end;
   end;
%mend LNG_rep;   


/*****************************************************************************************
 *   Needed for: %orva_cdfplot     ~line 170
 *               %orva_gfit        ~line 188
 *               %orva_histplot    ~line 176
 *               %orva_probplot    ~line 160
 *
 *   What it does  :  Evaluate the cumulative distribution function (CDF)
 *
 *   Parameters    :
 *              x     = value at which the CDF is evaluated
 *        returnVal = name of variable inton which result is stored
 *
 *****************************************************************************************/
%macro CdfLNG(x, returnVal);
    %if %sysevalf(&param_3 < 1e-8, boolean) %then %do;
        &returnVal=cdf("LOGNORMAL",&x,&param_1,&param_2);   
    %end;
    %else %do;
        &returnVal = 0;
        retain q&returnVal.1-q&returnVal&LNG_NQUANT;
        array q&returnVal q&returnVal.1-q&returnVal&LNG_NQUANT; 

        if _N_ = 1 then do;
            do i=1 to &LNG_NQUANT;
                if &Param_3 < 0.00002 then do;
                    q&returnVal[i] = sqrt(1 + probit(i/(&LNG_NQUANT+1)) * sqrt(&Param_3));
                end;
                else do;
                    q&returnVal[i] = sqrt(gaminv(i/(&LNG_NQUANT+1),1/&Param_3) * &Param_3);
                end;
            end;
        end; 

        do i=1 to &LNG_NQUANT;
            if &param_2*q&returnVal[i] < 1e-8 then do;
                &returnVal = &returnVal + 1;
            end;
            else do;
                &returnVal = &returnVal + cdf('LOGNORMAL',&x,&param_1,&param_2*q&returnVal[i]);
            end; 
        end;

        &returnVal=&returnVal/&LNG_NQUANT;
    %end;
%mend CdfLNG;


/*****************************************************************************************
 *   Needed for: %orva_loss_sim    ~line 263
 *
 *   What it does  :  Evaluate the survival distribution function (SDF)
 *
 *   Parameters    :
 *              x     = value at which the SDF is evaluated
 *        returnVal = name of variable inton which result is stored
 *
 *****************************************************************************************/
%macro SdfLNG(x, returnVal);
        %if %sysevalf(&param_3<1e-8,boolean) %then %do;
                &returnVal = 1- cdf("LOGNORMAL",&x,&param_1,&param_2);   
        %end;
        %else %do;
                &returnVal = 0;
                retain q1-q&LNG_NQUANT;
                array q q1-q&LNG_NQUANT; 

                if _N_ = 1 then do;
                        do i=1 to &LNG_NQUANT;
                                if &Param_3 < 0.00002 then do;
                                        q[i] = sqrt(1 + probit(i/(&LNG_NQUANT+1)) * sqrt(&Param_3));
                                end;
                                else do;
                                        q[i]=sqrt(gaminv(i/(&LNG_NQUANT+1),1/&Param_3) * &Param_3);
                                end;
                        end;
                end; 

                do i=1 to &LNG_NQUANT;
                        if &Param_2*q[i]<1e-8 then do;
                                &returnVal = &returnVal + 1;
                        end;
                        else do;
                                &returnVal = &returnVal + cdf('LOGNORMAL',&x, &Param_1, &Param_2*q[i]);
                        end; 
                end; 

                &returnVal = 1 - &returnVal/&LNG_NQUANT;
        %end;
%mend SdfLNG;

/*****************************************************************************************
 *   Needed for: %orva_histplot    ~line 176
 *
 *   What it does  :  Evaluate the density distribution function (CDF)
 *
 *   Parameters    :
 *              x     = value at which the density is evaluated
 *        returnVal = name of variable inton which result is stored
 *
 *****************************************************************************************/
%macro PdfLNG(x, returnVal);
        %if %sysevalf(&Param_3<1e-8, boolean) %then %do;
                &returnVal=pdf("LOGNORMAL",&x,&param_1,&param_2);   
        %end;
        %else %do;
                &returnVal = 0;
                retain q1-q&LNG_NQUANT;
                array q q1-q&LNG_NQUANT; 

                if _N_ = 1 then do;
                    do i=1 to &LNG_NQUANT;
                        if &Param_3 < 0.00002 then do;
                                q[i] = sqrt(1 + probit(i/(&LNG_NQUANT+1)) * sqrt(&Param_3));
                        end;
                        else do;
                                q[i] = sqrt(gaminv(i/(&LNG_NQUANT+1),1/&Param_3) * &Param_3);
                        end;
                    end;
                end; 
                        
                do i=1 to &LNG_NQUANT;
                        &returnVal = &returnVal + pdf('LOGNORMAL',&x,&param_1,&param_2*q[i]);
                end; 

                &returnVal=&returnVal/&LNG_NQUANT;
        %end;
%mend PdfLNG;

/*****************************************************************************************
 *   Needed for: %orva_quantplot    ~line 118
 *
 *   What it does  :  Generates a dataset named _temp_ containing probability values
 *                    and their corresponding quantiles
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro QuantLNG;
    %orvd_LOGN;
    %QuantLOGN;
    %orvd_LNG;

    %if %sysevalf( &param_3 > 1.0e-8, boolean) %then %do;
    data _parm_;
            mu=&param_1;
            sigma=&param_2;
            beta=&param_3;
    run;
    
        data _temp_;
            set _temp_; 
            quant = log(quant);
        run;

    proc model data=_temp_ parmsdata=_parm_ out=_temp1_ noprint;
        parms mu sigma beta;
        F=0;
        T=0;

        do i=1 to &LNG_NQUANT;
            q = sqrt(gaminv(i/(&LNG_NQUANT+1),1/beta) * beta);
                F = F + probnorm((quant-mu)/sigma/q)/&LNG_NQUANT;

            if THRESHOLD NE 0 then do;
                    T = T + probnorm((log(THRESHOLD)-mu)/sigma/q)/&LNG_NQUANT;
            end;
        end;

        F = (F-T)/(1-T);
        eq.one = F - ecdf;        
        solve quant/converge=1e-6;
    run;
    quit;   

    data _temp_;
        merge _temp_ _temp1_(keep=ecdf quant);
        quant=exp(quant);
    run;
    %end;
%mend QuantLNG;


/*****************************************************************************************
 *   Needed for: %orva_loss_sim    ~line 408
 *
 *   What it does  :  Simulates a random value
 *
 *   No Parameters    :  
 *
 *****************************************************************************************/
%macro SimLNG(loss);
        /*        
        This fix is done here because when the "param_3" parameter becomes too small,
        the simulation of the gamma variate fails. However this corresponds to a very
        peaky distribution around 1. Another way to look at this is that when the param_3 
        parameter is so small, the distribution degenerates to a log-normal and that is
        the choice that should have been made if one had used the AIC or BSC instead of 
        simply the best likellihood.
        */
        if (Param_3 < 0.000000001) then do;
                /*         Take absolut value here because we need to take a square
                        root later. This introduces a minimal bias that can be
                        ignored because the probability of this ever occuring 
                        is extremely low */
                gamma = abs(rand("NORMAL")*sqrt(Param_3) + 1.0);
        end;
        else do;
                gamma = rand("GAMMA",1/Param_3) * Param_3;
        end;

        &loss = rand("NORMAL") * Param_2 * sqrt(gamma) + Param_1;
        &loss = exp(&loss);
%mend SimLNG;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro InitFitLNG;
    %let nparms= 3;
    %let param_1_init= .;
    %let param_2_init= .;
    %let param_3_init= .;

    /*- initial guess based on moments -*/
    data _null_;
        mu    = &sample_log_mean;        
        beta  = &sample_log_kurtosis/3-1;
        sigma = sqrt(&sample_log_variance);

        call symput("param_1_init",mu);
        call symput("param_2_init",sigma);
        call symput("param_3_init",beta);        
    run;
        
    %if %sysevalf(&param_3_init < 0, boolean) %then %do;
            %let param_3_init=0.01;
    %end;
    
    %put **********************************;
    %put Distribution    =  &dist;
    %put Observations    =  &obsCount;
    %put Param_1_init    =  &Param_1_init;
    %put Param_2_init    =  &Param_2_init;
    %put Param_3_init    =  &Param_3_init;
    %put **********************************;
        

    %if %sysevalf(&obsCount > 5000, boolean) %then %do;
            %let sample_size=400;

            data _workdata7_; set _LogWorkdata_;
                    if ( ranuni(348975) < &sample_size/&obscount )then output;
            run;

            %FitLNG( inputdataset=_workdata7_ );
            
            data _NULL; set _parmnlp_(where=(_TYPE_="PARMS"));
              call symput("param_1_init",param_1);
              call symput("param_2_init",param_2);
              call symput("param_3_init",param_3);        
            run;

            %if %sysevalf(&param_3_init < 1.0e-2, boolean) %then %do;
              %let param_3_init=1.0e-2;
            %end;

	        %put **********************************;
	        %put Distribution    =  &dist;
	        %put Observations    =  &obsCount;
	        %put Param_1_init    =  &Param_1_init;
	        %put Param_2_init    =  &Param_2_init;
	        %put Param_3_init    =  &Param_3_init;
	        %put **********************************;
    %end;

%mend InitFitLNG;
       
/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC NLP
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro FitLNG( inputdataset=_LogWorkdata_);
        %let try=0;
        %let NLP_Tech=qn;                        /*quasi Newton-Raphson method */
        %let NLP_AltTech=newrap;        /* Newton-Raphson method */

        ods listing close;
        ods output 
                IterStop           =  _logl_
                ConvergenceStatus  =  _cs_ 
                covmat             = _covmat_ 
                ;        

%TRYAGAIN:

%if %sysevalf(&THRESHOLDPERCENT >= 0, boolean) %then %do;
            proc nlp 
                data   =  &inputdataset
                cov    =  4                pcov 
                maxit  =  10000 
                maxfu  =  10000 
                tech   =  &NLP_Tech 
                outest =  _parmnlp_
                ;

                parms 
                        Param_1 = &Param_1_init, 
                        Param_2 = &Param_2_init, 
                        Param_3 = &Param_3_init
                        ;

                bounds 
                        Param_3 > 0,
                        Param_2 > 0
                        ;

                retain q1-q&lng_nquant;
                array q q1-q&lng_nquant;                

                max logf;

                if _obs_=1 then do;
                    /*- generate the qi's */
                    %LNG_rep;
                end; 

                lcdf = 0;
                if param_3 < 1e-8 then do;
                      lcdf= &THRESHOLDPERCENT/(1-&THRESHOLDPERCENT)* 
                            logcdf("LOGNORMAL",THRESHOLD,param_1,param_2);
                end;
                else do;                    
                      do i=1 to &LNG_NQUANT;
                        if param_2*q[i] < 1e-8 then 
                             lcdf = lcdf + 1;
                        else 
                             lcdf = lcdf + cdf('LOGNORMAL',THRESHOLD,param_1,param_2*q[i]);
                      end;
                      lcdf=lcdf/&LNG_NQUANT;
                      lcdf= &THRESHOLDPERCENT/(1-&THRESHOLDPERCENT)* log(lcdf);
                end;
               
                logf=0;
                const = 1/sqrt(2*3.14159265358979323846264338328);

                do i=1 to &lng_nquant;
                   logf = logf + const * exp( -(LogAMOUNT - Param_1)**2/(2*(Param_2*q[i])**2) )
                                                / (Param_2*q[i]);
                end;

                logf = log(logf/&LNG_NQUANT) +lcdf;
        run;
        quit;
%end;
%else %do;

         proc nlp 
		data   =  &inputdataset
		cov    =  4		pcov 
		maxit  =  10000 
		maxfu  =  10000 
		tech   =  &NLP_Tech 
		outest =  _parmnlp_
		;

		parms 
			Param_1 = &Param_1_init, 
			Param_2 = &Param_2_init, 
			Param_3 = &Param_3_init
			;

		bounds 
			Param_3 > 0,
			Param_2 > 0
			;

		retain q1-q&lng_nquant;
		array q q1-q&lng_nquant;		

			/*If THRESHOLD is given, that means that we use
			 the same fixed threshold for all data points,
			 otherwise, we use the threshold value given
			 as argument*/
			 
		%if THRESHOLD ^= %str() %then %do;
			/*Fixed threshold so we can calculate it only once and save time*/
			retain logt; 
		%end;
			%else %do;
			/*Reinitialize at 0 for every data point because threshold might be different*/
			logt=0;     
			%end;

        max logf;

			if _obs_=1 then do;

				/*- generate the qi's */
	   			%LNG_rep;

				%if THRESHOLD ^= %str() %then %do;
					logt=0;
					if THRESHOLD ^= 0 then do; 
			  	       do i=1 to &lng_nquant;
	        			  logt = logt + probnorm((log(THRESHOLD)-param_1)/param_2/q{i});
			   	       end;
					end;
		   		%end;	   
    		end;	   

			logf=0;
	                const = 1/sqrt(2*3.14159265358979323846264338328);

  			do i=1 to &lng_nquant;
                                logf = logf + const *
						exp( -(LogAMOUNT - Param_1)**2/(2*(Param_2*q[i])**2) )
						/ (Param_2*q[i])
						;

				%if THRESHOLD=%str() %then %do;
					if THRESHOLD ^= 0 then do;
						logt = logt + probnorm((log(THRESHOLD)-param_1)/param_2/q{i});
					end;
				%end;
			end;

			logf = log(logf) - log(&LNG_NQUANT - logt);
        run;
        quit;

%end;
data _NULL_; set _parmnlp_ (where =(index(_type_,"PARMS")>0));
put param_1= param_2= param_3=;
run;

                data _null_;
                        set _cs_;
                        call symput("status",status);
                run;

                %if &status^=0  %then %do;
                        %if &try=0 %then %do;
                                %put ERROR: Error with &NLP_Tech method;
                                %put Trying with &NLP_AltTech method instead;
                                %let NLP_Tech = &NLP_AltTech;
                                %let try=1;
                                ods output 
                                                IterStop           =  _logl_
                                                ConvergenceStatus  =  _cs_ 
                                                covmat             =  _covmat_
                                                ;        
                                %goto TRYAGAIN;
                        %end;
                %end;

                ods listing;
%mend FitLNG;


/*****************************************************************************************
 *   Needed for: %orva_sfit_kri    ~line 304
 *
 *   What it does  :  Actually does the minimization using PROC MODEL
 *
 *   No Parameters    
 *
 *****************************************************************************************/
%macro FitKriLNG( inputdataset=_LogWorkdata_KRI_);
        %let try=0;
        %let NLP_Tech=qn;                        /*quasi Newton-Raphson method */
        %let NLP_AltTech=newrap;        /* Newton-Raphson method */

        ods listing close;
    ods output 
                IterStop           =  _logl_
                ConvergenceStatus  =  _cs_ 
                covmat             = _covmat_ 
                ;        

%TRYAGAIN:
            proc nlp 
                data   =  &inputdataset
                cov    =  4                pcov 
                maxit  =  10000 
                maxfu  =  10000 
                tech   =  &NLP_Tech 
                outest =  _parmnlp_
                ;

                parms 
                        Param_1 = &Param_1_init, 
                        Param_2 = &Param_2_init, 
                        Param_3 = &Param_3_init
                        ;

                bounds 
                        Param_3 > 0,
                        Param_2 > 0
                        ;

                retain q1-q&lng_nquant;
                array q q1-q&lng_nquant;                

                        /*If THRESHOLD is given, that means that we use
                         the same fixed threshold for all data points,
                         otherwise, we use the threshold value given
                         as argument*/
                         
                %if THRESHOLD ^= %str() %then %do;
                        /*Fixed threshold so we can calculate it only once and save time*/
                        retain logt; 
                %end;
                %else %do;
                        /*Reinitialize at 0 for every data point because threshold might be different*/
                        logt=0;     
                %end;

                max logf;

                if _obs_=1 then do;

                                /*- generate the qi's */
                    %LNG_rep;

                    %if THRESHOLD ^= %str() and %sysevalf(&THRESHOLDPERCENT < 0, boolean)  %then %do;
                        logt=0;
                        if THRESHOLD ^= 0 then do; 
                            do i=1 to &lng_nquant;
                                logt = logt + probnorm((log(THRESHOLD)-param_1 - KRI_LOG)/param_2/q{i});
                            end;
                        end;
                    %end;           
                end; 

                lcdf = 0;
                %if %sysevalf(&THRESHOLDPERCENT >= 0, boolean) %then %do;
                    logt = 0;
                    if param_3 < 1e-8 then do;
                      lcdf= &THRESHOLDPERCENT/(1-&THRESHOLDPERCENT)* 
                            logcdf("LOGNORMAL",THRESHOLD,param_1 + KRI_LOG,param_2);
 
                    end;
                    else do;                    
                      do i=1 to &LNG_NQUANT;
                        if param_2*q[i] < 1e-8 then 
                             lcdf = lcdf + 1;
                        else 
                             lcdf = lcdf + cdf('LOGNORMAL',THRESHOLD,param_1 + KRI_LOG,param_2*q[i]);
                      end;

                      lcdf=lcdf/&LNG_NQUANT;
                      lcdf= &THRESHOLDPERCENT/(1-&THRESHOLDPERCENT)* log(lcdf);

                    end;
                %end;
        

                logf=0;
                const = 1/sqrt(2*3.14159265358979323846264338328);

                do i=1 to &lng_nquant;
                   logf = logf + const * exp( -(LogAMOUNT - Param_1 - KRI_LOG)**2/(2*(Param_2*q[i])**2) )
                                                / (Param_2*q[i]);

                   %if THRESHOLD=%str() and %sysevalf(&THRESHOLDPERCENT < 0, boolean) %then %do;
                        if THRESHOLD ^= 0 then
                             logt=logt+probnorm((log(THRESHOLD)-param_1 - KRI_LOG)/param_2/q{i});
                                    
                   %end;
                end;

                logf = log(logf) - log(&lng_nquant-logt) + lcdf;
        run;
        quit;



                data _null_;
                        set _cs_;
                        call symput("status",status);
                run;

                %if &status^=0  %then %do;
                        %if &try=0 %then %do;
                                %put ERROR: Error with &NLP_Tech method;
                                %put Trying with &NLP_AltTech method instead;
                                %let NLP_Tech = &NLP_AltTech;
                                %let try=1;
                                ods output 
                                                IterStop           =  _logl_
                                                ConvergenceStatus  =  _cs_ 
                                                covmat             =  _covmat_
                                                ;        
                                %goto TRYAGAIN;
                        %end;
                %end;
                
                ods listing;
%mend FitKriLNG;

/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 320
 *
 *   What it does  :  Modify loglikelihood for cases when we had to take the log of the 
 *                                          observations first. Here, it does nothing.
 *
 *   NO Parameters    
 *
 *****************************************************************************************/
%macro MakeLLGoodLNG;
    %let sumlog= .;
    data _null_;
        set _workdata_ end=eof;
        retain sumlog 0;
        sumlog=sumlog+log(AMOUNT);
        call symput("sumlog",sumlog);
    run;        
%mend MakeLLGoodLNG;
%macro MakeLLGoodKriLNG;
    %let sumlog= .;
    data _null_;
        set _LogWorkdata_KRI_ end=eof;
        retain sumlog 0;
        sumlog=sumlog+log(AMOUNT);
        call symput("sumlog",sumlog);
    run;        
%mend MakeLLGoodKriLNG;


/*****************************************************************************************
 *   Needed for: %orva_sfitall    ~line 379
 *
 *   What it does  :  Checks whether some moments are missing.
 *                                          Here does nothing because all moments exist for Exponential distribution
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro NoMissingMOMLNG(parmdata=);
        data &parmdata;
      set &parmdata;
                
                if mean=. then do;
          status=1;
                        reason="Bad Parameter Combination: Mean does not exist.";
                end;
                else if std=. then do;
                        status=2;
                        reason="Bad Parameter Combination: Variance does not exist.";
      end;

        call symput("status",status);
   run;
%mend NoMissingMOMLNG;

/*****************************************************************************************
 *   Needed for: orva_sfitall_model.sas
 *                                                                %FindMoments ~line 392
 *
 *   What it does  :  Calculates the moments
 *
 *   No Parameters 
 *
 *****************************************************************************************/
/*- actually, the moments of logarithm of losses -*/
%macro MomentsLNG;
        m1=.;
        m2=.;
        m3=.;
        m4=.;
    mean=param_1;
    std=param_2;
    skewness=0;
    kurtosis=3*(param_3+1);
%mend MomentsLNG;

%macro RealMomentsLNG;
    %do i=1 %to 4;
        if param_3 * param_2 * param_2 < 2/(&i * &i) then do;
            if Param_3 < 1.0e-8 then do;
                m&i = exp(&i * param_1+ &i * &i * param_2 * param_2 / 2);
            end;
            else do;
            m&i = exp(&i * param_1) * (1-param_3 * &i * &i * param_2 * param_2 / 2)**(-1/param_3);
            end;
        end;
        else do;
            m&i = .;
        end;
    %end;  

    MEAN     = m1;
    STD      = sqrt(m2-m1*m1);
    SKEWNESS = (m3-3*m1*m2+2*(m1**3))/(std**3);
    KURTOSIS = (m4-4*m1*m3+6*(m1**2)*m2-3*(m1**4))/(std**4);
%mend RealMomentsLNG;


/*****************************************************************************************
 *   Needed for: %orva_sfitall_mom.sas      ~line 78
 *
 *   What it does  :  Does method of moments estimation and assigns the corresponding 
 *                    values to the param_1, param_2, param_3, and shape2 macros
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro MomFitLNG;
    %let param_1= .;
    %let param_2= .;
    %let param_3= .;        
    data _null_;
       p2 = sqrt(&variance);
       p3 = &kurtosis/3 - 1;
       call symput("param_1",&mean);
       call symput("param_2",p2);
       call symput("param_3",p3);
    run;
%mend MomFitLNG;

%macro LNG_unboundedmoments;
       %do i=1 %to 4;
          if param_3 * param_2 * param_2 < 2/(&i * &i) then do;
            
            m&i = (&i * param_1) + log(1-param_3 * &i * &i * param_2 * param_2 / 2)*(-1/param_3); 
            m&i = exp(m&i); 
        end;
        else do;
            m&i = .;
        end;
     %end;  

    MEAN     = m1;
    STD      = sqrt(m2-m1*m1);
    SKEWNESS = (m3-3*m1*m2+2*(m1**3))/(std**3);
    KURTOSIS = (m4-4*m1*m3+6*(m1**2)*m2-3*(m1**4))/(std**4);
%mend LNG_unboundedmoments;

%macro MomFit_real_LNG;
    %let param_1_init= .;
    %let param_2_init= .;
    %let param_3_init= .;
    data _null_;
        c = &mean**2+&variance;
    	
    	if c <= 0 then do; 
    		p1 = .;
    		p2 = .;
			call symput("status",1);
			call symput("reason","Unable to compute MOM estimates");
	end;
	else do;
		c = sqrt(c);
		p1 = log(&mean**2/c);
		p2 = sqrt(2*log(c/&mean));
	end;
		
	
        param_3 = .5 /( 8 * p2**2);
        call symput("param_1_init",p1);
        call symput("param_2_init",p2);
        call symput("param_3_init",param_3);
    run;

    
    
    ods listing;
    proc nlp  maxit=10000 maxfu=10000 tech=tr outest=_parmnlp_ ;
        min obj;
        parms param_1=&param_1_init,
              param_2=&param_2_init,
              param_3=&param_3_init;
        bounds param_1>0, param_2>0, param_3>1.0e-5;

        %LNG_unboundedmoments;       
       
        f1 = (mean-&mean);
        f2 = (std*std-&variance);
        f4 = (kurtosis-&kurtosis);
        obj = f1**2 +sqrt(f2**2) + (f4**2)**(.25);
    run; quit;

    data _null_;
        set _parmnlp_;
        if _TYPE_ = "PARMS" then do;
             call symput("param_1",param_1);
             call symput("param_2",param_2);
             call symput("param_3",param_3);
        end;
    run;  
    %if &SYSERR^=0 & &SYSERR ^= 4 %then %do;
        %put ERROR: Unable to find moments estimates for LNG.;
        %let status=1;
        %let reason=Unable to find moments estimates for LNG.;
    %end;


%mend MomFit_real_LNG;

/*****************************************************************************************
 *   Needed for: %orvr_sfitstats.sas      ~line 78
 *
 *   What it does  :  Displays the estimated parameters
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro ParamsLNG;
    data foo (keep=Variable Estimate StdErr TValue probt);
            format Variable $12.;
            set &sparm;
            Variable="Location";
            Estimate=param_1;
            StdErr=sqrt(cov_1);
            TValue=estimate/StdErr;
            probt=(1-probt(abs(tvalue),obscount -&KRI_ADJUST-&Param_count))*2;
            output;
            Variable="Scale";
            Estimate=param_2;
            StdErr=sqrt(cov_3);
            TValue=estimate/StdErr;
            probt=(1-probt(abs(tvalue),obscount -&KRI_ADJUST-&Param_count))*2;
            output;
            Variable="Shape1";
            Estimate=param_3;
            StdErr=sqrt(cov_6);
            TValue=estimate/StdErr;
            probt=(1-probt(abs(tvalue),obscount -&KRI_ADJUST-&Param_count))*2;
            output;
    run;

    data _NULL_; set foo;
      file print ods=(template='Stat.Reg.ParameterEstimates');
      put _ods_;
    run;
%mend ParamsLNG;

/*****************************************************************************************
 *   Needed for: %orvr_sfitstats.sas      ~line 78
 *
 *   What it does  :  Displays the estimated parameters
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro CovmatLNG;
    data foo (keep=_name_ param_1 param_2 param_3 );
        format _name_ $12.;
        set &sparm;
        _name_="Location";
        param_1 = cov_1;
        param_2 = cov_2;
        param_3 = cov_4;
        output;
        _name_="Scale";
        param_1 =  cov_2;
        param_2 =  cov_3;
        param_3 =  cov_5;
        output;
        _name_="Shape1";
        param_1 = cov_4;
        param_2 = cov_5;
        param_3 = cov_6;
        output;
    run;

    proc print data=foo noobs label;
        var _name_ param_1 param_2 param_3 ;
        label _name_='00'x param_1="Location" param_2="Scale" param_3="Shape1";
    run;
%mend CovmatLNG;


/*****************************************************************************************
 *   What it does  :  Calculates the loglikelihood contribution for each observation
 *
 *   No Parameters 
 *
 *****************************************************************************************/
%macro LLLNG(var,retVal);
    if _obs_=1 then do;
        /*- generate the qi's */
        %LNG_rep;
        %if "&THRESHOLD" ^= "" %then %do;
            logt=0;
            if THRESHOLD ^= 0 then do; 
                do i=1 to &LNG_NQUANT;
                    logt = logt + probnorm((log(THRESHOLD)-param_1)/param_2/q{i});
                end;
            end;
        %end;           
    end;           

    logf=0;

    do i=1 to &LNG_NQUANT;
        &retVal = &retVal + pdf("NORMAL",AMOUNT,param_1,param_2*q{i});
        %if "&THRESHOLD"="" %then %do;
            if THRESHOLD ^= 0 then do;
                logt=logt+probnorm((log(THRESHOLD)-param_1)/param_2/q{i});
            end;
        %end;
    end;

    &retVal = log(&retVallogf) - log(&LNG_NQUANT-logt);
%mend LLLNG;

/*****************************************************************************************
 *   Needed to be called before anything else is available
 *****************************************************************************************/
%macro ORVD_LNG;   
    %GLOBAL LNG_NQUANT;
    %let EXTRA_PARAMS= ;
    %let NEEDED_INPUT= ;
    %let SCALE_TRANSFORM= LOG;
    %let DIST_NAME= Log Normal Gamma;
    %let PARAM_COUNT= 3;
    %let LNG_NQUANT= 31;

    data LNG_STATUS;
        status= 0;       
    run;
%mend ORVD_LNG;

