%macro orvl_cmn_cpi(INLIB, OUTLIB, DATA_VERSION_ID, SOURCE_TYPE);


   %let TYP_NM = CPIs;

   /* 
    * *************************************************
    * Make sure the required dataset has records. 
    * If it doesn't, orvl_ds_exists sets LOAD_STATUS to 2.
    * *************************************************
    */
   %orvl_ds_exists(&INLIB..cpi, "&TYP_NM", false);
   %if &ds_nobs eq 0 %then %do;
      %orvl_cmn_status_success;
      %return;
   %end;

   %let MOD_CNT = 10;
   
   %put STATUS PROGRESS2 TOTAL=&ds_nobs FILE="&TYP_NM";

   /*
    * ****************************
    * Process the CPI items. 
    * ****************************
    */
   /* Get the starting uid for items to insert. */
   %orvdb_seq_incrementer(&SEQ_LIB, LOAD_CPI_SEQ, dbload, INC=&ds_nobs);
   proc sql;
      select VALUE Into :THE_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;

   /* Create a temporary dataset with the new ids. */
	data WORK.TMP_CPI (KEEP= CPI_UID 
                            FEED_REF_ID 
                            CPI_COUNTRY_ID 
                            CPI_YEAR 
                            CPI_VALUE 
                            CPI_SOURCE 
                            VERSION_UID)
        WORK.TMP_LOAD_STATUS_DETAILS_CPI (KEEP= WHO
                                                STATUS 
                                                REASON);
		set &INLIB..cpi (rename=(CPI_ID=in_id
                               Country_ID=in_country_id
                               Year=in_year
                               Value=in_value));
      length WHO $32. REASON $200.;
		retain id_val;

		if _N_ = 1 then do;
			id_val = symgetn('THE_ID');
			if (id_val eq .) then do;
				id_val = 1;
			end;
         declare hash ctryHash(dataset: 'WORK.TMP_COUNTRY_INT', hashexp: 10);
         rc = ctryHash.defineKey('FEED_REF_ID');
         rc = ctryHash.defineData('COUNTRY_UID');
         rc = ctryHash.defineDone();
         call missing(COUNTRY_UID, FEED_REF_ID);
		end;
		else do;
			id_val = id_val + 1;
		end;

      if (mod(_N_,&MOD_CNT) eq 0) then do;
         put "STATUS PROGRESS2 COUNT_INC=&MOD_CNT";
         %orvl_cmn_user_cancelled_check();
         status_cancel = symgetn('ORVAR_CANCEL');
         if status_cancel ne 0 then stop;
      end;

      /* Assign whatever variables are available. */
		CPI_UID = id_val;
      CPI_YEAR = in_year;
      CPI_VALUE = in_value;
      CPI_SOURCE = &SOURCE_TYPE;
		VERSION_UID = &DATA_VERSION_ID;

      /* Create a flag that will be used to determine if every item was found. */
      doOutPut = 1;

      /* Find the Country item. */
      FEED_REF_ID = in_country_id;
      rc = ctryHash.find();
      %orvl_find_reference(rc, "&TYP_NM - Country", WORK.TMP_LOAD_STATUS_DETAILS_CPI);
      CPI_COUNTRY_ID = COUNTRY_ID;

      FEED_REF_ID = in_id;

      %orvl_cmn_test_required_value(FEED_REF_ID, "CPI_ID", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_CPI);

      /* Output the item only if all of the referenced components were found. */
      if doOutPut eq 1 then do;
         output WORK.TMP_CPI;
      end;
	run;

   %orvl_cmn_user_cancelled_ck_wrap();
   %if &ORVAR_CANCEL ne 0 %then %goto DONE;

   %orvl_cmn_increment_remaining(&ds_nobs,&MOD_CNT);

   %orvl_ds_exists(WORK.TMP_LOAD_STATUS_DETAILS_CPI, "&TYP_NM", false);
   %if &ds_nobs ne 0 %then %do;
      %put STATUS - Appending Error/Warning items - This may take a while.;
      proc append base=WORK.LOAD_STATUS_DETAILS data=WORK.TMP_LOAD_STATUS_DETAILS_CPI;
      run;
   %end;

   /* Append the temp datasets to the original datasets. */
   %if %TRIM(%LEFT(&LOAD_STATUS)) eq 1 %then %do;
      %orvl_cmn_proc_append(&TYP_NM, &OUTLIB..CPI, WORK.TMP_CPI);
      %orvl_cmn_user_cancelled_ck_wrap();
      %if &ORVAR_CANCEL ne 0 %then %goto DONE;
   %end;

%DONE:
   %put STATUS PROGRESS2 DONE;

   /* Handle the final signaling. */
   %orvl_cmn_status_processing;

%mend orvl_cmn_cpi;
