%macro orvl_int_kris(INLIB, OUTLIB, DATA_VERSION_ID);


   %let TYP_NM = Risk Factors;
   
   /* 
    * *************************************************
    * Make sure the required dataset has records. 
    * If it doesn't, orvl_ds_exists sets LOAD_STATUS to 2.
    * *************************************************
    */
   %orvl_ds_exists(&INLIB..risk_factor_variable, "&TYP_NM", false);
   %if &ds_nobs eq 0 %then %do;
      %orvl_cmn_status_success;
      %return;
   %end;

   %put STATUS PROGRESS2 TOTAL=&ds_nobs FILE="&TYP_NM";

   /*
    * *******************************
    * Process the Risk Factor items. 
    * *******************************
    */
   /* Get the starting uid for items to insert. */
   %orvdb_seq_incrementer(&SEQ_LIB, LOAD_Risk_Factor_Var_SEQ, dbload, INC=&ds_nobs);
   proc sql;
      select VALUE Into :THE_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;

   /* Create temporary datasets, with the new ids. */
   data WORK.TMP_RISK_FACTOR_VARIABLE (KEEP= RISK_FACTOR_VARIABLE_UID 
                                             FEED_REF_ID 
                                             RISK_FACTOR_VARIABLE_NAME
                                             UPPER_BOUND 
                                             LOWER_BOUND 
                                             VERSION_UID)
        WORK.TMP_LOAD_STATUS_DETAILS_KRI (KEEP= WHO
                                                STATUS 
                                                REASON);
      set &INLIB..risk_factor_variable (rename=(Risk_Factor_Variable_ID=in_id
                                                Risk_Factor_Variable_Name=in_name));
      length RISK_FACTOR_VARIABLE_NAME $100.;
      length WHO $32. REASON $200.;

      retain id_val;

      if _N_ = 1 then do;
         id_val = symgetn('THE_ID');
         if (id_val eq .) then do;
            id_val = 1;
         end;
      end;
      else do;
         id_val = id_val + 1;
      end;

      put "STATUS PROGRESS2 COUNT_INC";
      %orvl_cmn_user_cancelled_check();
      status_cancel = symgetn('ORVAR_CANCEL');
      if status_cancel ne 0 then stop;

      /* Assign whatever variables are available. */
      RISK_FACTOR_VARIABLE_UID = id_val;
      FEED_REF_ID = in_id;
      RISK_FACTOR_VARIABLE_NAME = in_name;
      VERSION_UID = &DATA_VERSION_ID;
      
      /* Create a flag that will be used to determine if every item was found. */
      doOutput = 1;

      %orvl_cmn_test_required_value(FEED_REF_ID, "Risk_Factor_Variable_ID", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_KRI);
      %orvl_cmn_test_required_string(RISK_FACTOR_VARIABLE_NAME, "Risk_Factor_Variable_Name", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_KRI);

      if doOutPut eq 1 then do;
         output WORK.TMP_RISK_FACTOR_VARIABLE;
      end;
   run;

   %orvl_cmn_user_cancelled_ck_wrap();
   %if &ORVAR_CANCEL ne 0 %then %goto DONE;

   %orvl_ds_exists(WORK.TMP_LOAD_STATUS_DETAILS_KRI, "&TYP_NM", false);
   %if &ds_nobs ne 0 %then %do;
      %put STATUS - Appending Error/Warning items - This may take a while.;
      proc append base=WORK.LOAD_STATUS_DETAILS data=WORK.TMP_LOAD_STATUS_DETAILS_KRI;
      run;
   %end;

   /* Flags for appending bl and firm. */
   data _null_;
      call symput('DO_BL', 0);
      call symput('DO_FIRM', 0);
   run;
   
   %let TYP_NM2 = Risk Factor BLs;

   /* 
    * *************************************************
    * Make sure the required dataset has records. 
    * If it doesn't, orvl_ds_exists sets LOAD_STATUS to 2.
    * *************************************************
    */
   %orvl_ds_exists(&INLIB..risk_factor_value_business_line, "&TYP_NM2", false);
   %if &ds_nobs eq 0 %then %do;
      %goto SKIPPER1;
   %end;

   %let MOD_CNT = 10;
   
   %put STATUS PROGRESS2 TOTAL=&ds_nobs FILE="&TYP_NM2";

   /*
    * *********************************************
    * Process the Risk Factor Business Line items. 
    * *********************************************
    */
   /* Get the starting uid for items to insert. */
   %orvdb_seq_incrementer(&SEQ_LIB, LOAD_Risk_Factor_BL_SEQ, dbload, INC=&ds_nobs);
   proc sql;
      select VALUE Into :THE_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;

   /* Create temporary datasets, with the new ids. */
   data WORK.TMP_RISK_FACTOR_VALUE_BL (KEEP= RISK_FACTOR_VALUE_BL_UID 
                                             FEED_REF_ID 
                                             RISK_FACTOR_VARIABLE_UID 
                                             RISK_FACTOR_VARIABLE_DATE 
                                             INTERNAL_BL_UID 
                                             RISK_FACTOR_VALUE 
                                             VERSION_UID)
        WORK.TMP_LOAD_STATUS_DETAILS_RFBL (KEEP= WHO
                                                 STATUS 
                                                 REASON);
      set &INLIB..risk_factor_value_business_line (rename=(Risk_Factor_Value_BL_ID=in_id
                                                           Risk_Factor_Variable_ID=in_risk_factor_ref_id
                                                           Risk_Factor_Variable_Date=in_date
                                                           Internal_BL_ID=in_bl_ref_id
                                                           Risk_Factor_Variable_Value=in_value));
      length WHO $32. REASON $200.;

      retain id_val;

      if _N_ = 1 then do;
         id_val = symgetn('THE_ID');
         if (id_val eq .) then do;
            id_val = 1;
         end;
         declare hash kriHash(dataset: "WORK.TMP_RISK_FACTOR_VARIABLE", hashexp: 10);
         rc = kriHash.defineKey('FEED_REF_ID');
         rc = kriHash.defineData('RISK_FACTOR_VARIABLE_UID');
         rc = kriHash.defineDone();
         declare hash blHash(dataset: "WORK.TMP_INTERNAL_BL", hashexp: 10);
         rc = blHash.defineKey('FEED_REF_ID');
         rc = blHash.defineData('INTERNAL_BL_UID');
         rc = blHash.defineDone();
         call missing(FEED_REF_ID, RISK_FACTOR_VARIABLE_UID, INTERNAL_BL_UID);
         call symput('DO_BL', 1);
      end;
      else do;
         id_val = id_val + 1;
      end;

      if (mod(_N_,&MOD_CNT) eq 0) then do;
         put "STATUS PROGRESS2 COUNT_INC=&MOD_CNT";
         %orvl_cmn_user_cancelled_check();
         status_cancel = symgetn('ORVAR_CANCEL');
         if status_cancel ne 0 then stop;
      end;

      /* Assign whatever variables are available. */
      RISK_FACTOR_VALUE_BL_UID = id_val;
      RISK_FACTOR_VARIABLE_DATE = in_date;
      RISK_FACTOR_VALUE=in_value;
      VERSION_UID = &DATA_VERSION_ID;

      /* Create a flag that will be used to determine if every item was found. */
      doOutPut = 1;

      /* Find the Risk Factor Variable. */
      FEED_REF_ID = in_risk_factor_ref_id;
      rc = kriHash.find();
      %orvl_find_reference(rc, "&TYP_NM2 - R. Factor", WORK.TMP_LOAD_STATUS_DETAILS_RFBL);

      /* Find the Business Line. */
      FEED_REF_ID = in_bl_ref_id;
      rc = blHash.find();
      %orvl_find_reference(rc, "&TYP_NM2 - BL", WORK.TMP_LOAD_STATUS_DETAILS_RFBL);

      FEED_REF_ID = in_id;

      %orvl_cmn_test_required_value(FEED_REF_ID, "Risk_Factor_Value_BL_ID", _N_, "&TYP_NM2 - BL", WORK.TMP_LOAD_STATUS_DETAILS_RFBL);
      %orvl_cmn_test_required_value(RISK_FACTOR_VARIABLE_DATE, "Risk_Factor_Variable_Date", _N_, "&TYP_NM2 - BL", WORK.TMP_LOAD_STATUS_DETAILS_RFBL);
      %orvl_cmn_test_required_value(RISK_FACTOR_VALUE, "Risk_Factor_Variable_Value", _N_, "&TYP_NM2 - BL", WORK.TMP_LOAD_STATUS_DETAILS_RFBL);

      /* Output the item only if all of the referenced components were found. */
      if doOutPut eq 1 then do;
         output WORK.TMP_RISK_FACTOR_VALUE_BL;
      end;
   run;

   %orvl_cmn_user_cancelled_ck_wrap();
   %if &ORVAR_CANCEL ne 0 %then %goto DONE;

   %orvl_cmn_increment_remaining(&ds_nobs,&MOD_CNT);

   %orvl_ds_exists(WORK.TMP_LOAD_STATUS_DETAILS_RFBL, "&TYP_NM2", false);
   %if &ds_nobs ne 0 %then %do;
      %put STATUS - Appending Error/Warning items - This may take a while.;
      proc append base=WORK.LOAD_STATUS_DETAILS data=WORK.TMP_LOAD_STATUS_DETAILS_RFBL;
      run;
   %end;

%SKIPPER1:

   %let TYP_NM3 = Risk Factor Firms;
   
   /* 
    * *************************************************
    * Make sure the required dataset has records. 
    * If it doesn't, orvl_ds_exists sets LOAD_STATUS to 2.
    * *************************************************
    */
   %orvl_ds_exists(&INLIB..risk_factor_value_firm, "&TYP_NM3", false);
   %if &ds_nobs eq 0 %then %do;
      %goto SKIPPER2;
   %end;

   %put STATUS PROGRESS2 TOTAL=&ds_nobs FILE="&TYP_NM3";

   /*
    * *******************************
    * Process the Risk Factor items. 
    * *******************************
    */
   /* Get the starting uid for items to insert. */
   %orvdb_seq_incrementer(&SEQ_LIB, LOAD_Risk_Factor_Firm_SEQ, dbload, INC=&ds_nobs);
   proc sql;
      select VALUE Into :THE_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;

   /* Create temporary datasets, with the new ids. */
   data WORK.TMP_RISK_FACTOR_VALUE_FIRM (KEEP= RISK_FACTOR_VALUE_FIRM_UID 
                                               FEED_REF_ID 
                                               RISK_FACTOR_VARIABLE_UID 
                                               RISK_FACTOR_VARIABLE_DATE 
                                               RISK_FACTOR_VALUE 
                                               VERSION_UID)
        WORK.TMP_LOAD_STATUS_DETAILS_RFF (KEEP= WHO
                                                STATUS 
                                                REASON);
      set &INLIB..risk_factor_value_firm (rename=(Risk_Factor_Value_Firm_ID=in_id
                                                  Risk_Factor_Variable_ID=in_risk_factor_ref_id
                                                  Risk_Factor_Variable_Date=in_date
                                                  Risk_Factor_Variable_Value=in_value));
      length WHO $32. REASON $200.;
      length DUP_KEY $64.;

      retain id_val;

      if _N_ = 1 then do;
         id_val = symgetn('THE_ID');
         if (id_val eq .) then do;
            id_val = 1;
         end;
         declare hash kriHash(dataset: "WORK.TMP_RISK_FACTOR_VARIABLE", hashexp: 10);
         rc = kriHash.defineKey('FEED_REF_ID');
         rc = kriHash.defineData('RISK_FACTOR_VARIABLE_UID');
         rc = kriHash.defineDone();
         declare hash dupHash(hashexp: 10);
         rc = dupHash.defineKey('DUP_KEY');
         rc = dupHash.defineData('ITEM_EXISTS');
         rc = dupHash.defineDone();
         call missing(FEED_REF_ID, RISK_FACTOR_VARIABLE_UID, DUP_KEY, ITEM_EXISTS);
         call symput('DO_FIRM', 1);
      end;
      else do;
         id_val = id_val + 1;
      end;

      put "STATUS PROGRESS2 COUNT_INC";
      %orvl_cmn_user_cancelled_check();
      status_cancel = symgetn('ORVAR_CANCEL');
      if status_cancel ne 0 then stop;

      /* Assign whatever variables are available. */
      RISK_FACTOR_VALUE_FIRM_UID = id_val;
      RISK_FACTOR_VARIABLE_DATE = in_date;
      RISK_FACTOR_VALUE=in_value;
      VERSION_UID = &DATA_VERSION_ID;
      DUP_KEY = trim(left(put(in_risk_factor_ref_id, best.))) || '_' || trim(left(put(in_date, best.)));
      ITEM_EXISTS = 1;

      /* Create a flag that will be used to determine if every item was found. */
      doOutPut = 1;

      /* Find the Risk Factor Variable. */
      FEED_REF_ID = in_risk_factor_ref_id;
      rc = kriHash.find();
      %orvl_find_reference(rc, "&TYP_NM3 - R. Factor", WORK.TMP_LOAD_STATUS_DETAILS_RFF);

      FEED_REF_ID = in_id;

      %orvl_cmn_test_required_value(FEED_REF_ID, "Risk_Factor_Value_Firm_ID", _N_, "&TYP_NM3 - R. Factor", WORK.TMP_LOAD_STATUS_DETAILS_RFF);
      %orvl_cmn_test_required_value(RISK_FACTOR_VARIABLE_DATE, "Risk_Factor_Variable_Date", _N_, "&TYP_NM3 - R. Factor", WORK.TMP_LOAD_STATUS_DETAILS_RFF);
      %orvl_cmn_test_required_value(RISK_FACTOR_VALUE, "Risk_Factor_Variable_Value", _N_, "&TYP_NM3 - R. Factor", WORK.TMP_LOAD_STATUS_DETAILS_RFF);

      rc = dupHash.find();
      if (rc eq 0) then do;
         %orvl_cmn_set_reason_prefix(_N_);
         temp_str = "STATUS WARNING: " || trim(left(ORVAR_REASON_OBS_PREFIX)) || " The risk factor has been previously added and will be dropped.";
         put temp_str;
         doOutPut = 0;
      end;
      else do;
         rc = dupHash.add();
         if (rc ^= 0) then do;
            %orvl_cmn_set_reason_prefix(_N_);
            temp_str = "STATUS WARNING: " || trim(left(ORVAR_REASON_OBS_PREFIX)) || " The risk factor could not be added to the check-for-dups hash.";
            put temp_str;
         end;
      end;

      /* Output the item only if all of the referenced components were found. */
      if doOutPut eq 1 then do;
         output WORK.TMP_RISK_FACTOR_VALUE_FIRM;
      end;
   run;

   %orvl_cmn_user_cancelled_ck_wrap();
   %if &ORVAR_CANCEL ne 0 %then %goto DONE;

   %orvl_ds_exists(WORK.TMP_LOAD_STATUS_DETAILS_RFF, "&TYP_NM3", false);
   %if &ds_nobs ne 0 %then %do;
      %put STATUS - Appending Error/Warning items - This may take a while.;
      proc append base=WORK.LOAD_STATUS_DETAILS data=WORK.TMP_LOAD_STATUS_DETAILS_RFF;
      run;
   %end;

%SKIPPER2:

   /* Append the temp datasets to the original datasets. */
   %if %TRIM(%LEFT(&LOAD_STATUS)) eq 1 %then %do;
      %orvl_cmn_proc_append(&TYP_NM, &OUTLIB..RISK_FACTOR_VARIABLE, WORK.TMP_RISK_FACTOR_VARIABLE);
      %orvl_cmn_user_cancelled_ck_wrap();
      %if &ORVAR_CANCEL ne 0 %then %goto DONE;
      %if %TRIM(%LEFT(&DO_BL)) eq 1 and %TRIM(%LEFT(&LOAD_STATUS)) eq 1 %then %do;
         %orvl_cmn_proc_append(&TYP_NM2, &OUTLIB..RISK_FACTOR_VALUE_BL, WORK.TMP_RISK_FACTOR_VALUE_BL);
         %orvl_cmn_user_cancelled_ck_wrap();
         %if &ORVAR_CANCEL ne 0 %then %goto DONE;
      %end;
      %if %TRIM(%LEFT(&DO_FIRM)) eq 1 and %TRIM(%LEFT(&LOAD_STATUS)) eq 1 %then %do;
         %orvl_cmn_proc_append(&TYP_NM3, &OUTLIB..RISK_FACTOR_VALUE_FIRM, WORK.TMP_RISK_FACTOR_VALUE_FIRM);
         %orvl_cmn_user_cancelled_ck_wrap();
         %if &ORVAR_CANCEL ne 0 %then %goto DONE;
      %end;
   %end;

%DONE:
   %put STATUS PROGRESS2 DONE;

   /* Handle the final signaling. */
   %orvl_cmn_status_processing;

%mend orvl_int_kris;
