%macro orvl_int_scale(INLIB, OUTLIB, DATA_VERSION_ID);


   %let TYP_NM = Scale Factors;
   %let TYP_NM2 = SF BLs;
   %let TYP_NM3 = SF Firms;
   
   /* 
    * *************************************************
    * Make sure the required dataset has records. 
    * If it doesn't, orvl_ds_exists sets LOAD_STATUS to 2.
    * *************************************************
    */
   %orvl_ds_exists(&INLIB..scale_factor, "&TYP_NM", false);
   %if &ds_nobs eq 0 %then %do;
      %orvl_cmn_status_success;
      %return;
   %end;

   %put STATUS PROGRESS2 TOTAL=&ds_nobs FILE="&TYP_NM";

   /*
    * *******************************
    * Process the Scale Factor items. 
    * *******************************
    */
   /* Get the starting uid for items to insert. */
   %orvdb_seq_incrementer(&SEQ_LIB, LOAD_Scale_Factor_Var_SEQ, dbload, INC=&ds_nobs);
   proc sql;
      select VALUE Into :THE_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;

   /* Create a temporary dataset with the new ids. */
   data WORK.TMP_SF_VARIABLE (KEEP= SCALE_FACTOR_VARIABLE_UID 
                                    FEED_REF_ID 
                                    SCALE_FACTOR_VARIABLE_NAME 
                                    VERSION_UID)
        WORK.TMP_LOAD_STATUS_DETAILS_SFV (KEEP= WHO
                                                STATUS 
                                                REASON);
      set &INLIB..scale_factor (rename=(Scale_Factor_ID=in_id
                                        Scale_Factor_Name=in_name));
      length SCALE_FACTOR_VARIABLE_NAME $100.;

      retain id_val;

      if _N_ = 1 then do;
         id_val = symgetn('THE_ID');
         if (id_val eq .) then do;
            id_val = 1;
         end;
      end;
      else do;
         id_val = id_val + 1;
      end;

      put "STATUS PROGRESS2 COUNT_INC";
      %orvl_cmn_user_cancelled_check();
      status_cancel = symgetn('ORVAR_CANCEL');
      if status_cancel ne 0 then stop;

      /* Assign whatever variables are available. */
      SCALE_FACTOR_VARIABLE_UID = id_val;
      FEED_REF_ID = in_id;
      SCALE_FACTOR_VARIABLE_NAME = in_name;
      VERSION_UID = &DATA_VERSION_ID;
      
      /* Create a flag that will be used to determine if every item was found. */
      doOutput = 1;

      %orvl_cmn_test_required_value(FEED_REF_ID, "Scale_Factor_ID", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_SFV);
      %orvl_cmn_test_required_string(SCALE_FACTOR_VARIABLE_NAME, "Scale_Factor_Name", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_SFV);

      if doOutPut eq 1 then do;
         output WORK.TMP_SF_VARIABLE;
      end;
   run;

   %orvl_cmn_user_cancelled_ck_wrap();
   %if &ORVAR_CANCEL ne 0 %then %goto DONE;

   %orvl_ds_exists(WORK.TMP_LOAD_STATUS_DETAILS_SFV, "&TYP_NM2", false);
   %if &ds_nobs ne 0 %then %do;
      proc append base=WORK.LOAD_STATUS_DETAILS data=WORK.TMP_LOAD_STATUS_DETAILS_SFV;
      run;
   %end;

   /* Flags for appending bl and firm. */
   data _null_;
      call symput('DO_BL', 0);
      call symput('DO_FIRM', 0);
   run;
   
   /* 
    * *************************************************
    * Make sure the required dataset has records. 
    * If it doesn't, orvl_ds_exists sets LOAD_STATUS to 2.
    * *************************************************
    */
   %orvl_ds_exists(&INLIB..scale_factor_value_business_line, "&TYP_NM2", false);
   %if &ds_nobs eq 0 %then %do;
      %goto SKIPPER1;
   %end;

   %let MOD_CNT = 10;
   
   %put STATUS PROGRESS2 TOTAL=&ds_nobs FILE="&TYP_NM2";

   /*
    * *********************************************
    * Process the Scale Factor Business Line items. 
    * *********************************************
    */
   /* Get the starting uid for items to insert. */
   %orvdb_seq_incrementer(&SEQ_LIB, LOAD_Scale_Factor_BL_SEQ, dbload, INC=&ds_nobs);
   proc sql;
      select VALUE Into :THE_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;

   /* Create temporary datasets, with the new ids. */
   data WORK.TMP_SCALE_FACTOR_BL (KEEP= SCALE_FACTOR_BL_UID 
                                        FEED_REF_ID 
                                        SCALE_FACTOR_YEAR
                                        SCALE_FACTOR_VARIABLE_UID 
                                        INTERNAL_BL_UID 
                                        SCALE_FACTOR_VALUE 
                                        VERSION_UID)
        WORK.TMP_LOAD_STATUS_DETAILS_SFBL (KEEP= WHO
                                                 STATUS 
                                                 REASON);
      set &INLIB..scale_factor_value_business_line (rename=(Scale_Factor_Value_BL_ID=in_id
                                                            Scale_Factor_ID=in_sf_ref_id
                                                            Scale_Factor_Year=in_year
                                                            Internal_BL_ID=in_bl_ref_id
                                                            Scale_Factor_Value=in_value));
      length WHO $32. REASON $200.;

      retain id_val;

      if _N_ = 1 then do;
         id_val = symgetn('THE_ID');
         if (id_val eq .) then do;
            id_val = 1;
         end;
         declare hash sfHash(dataset: "WORK.TMP_SF_VARIABLE", hashexp: 10);
         rc = sfHash.defineKey('FEED_REF_ID');
         rc = sfHash.defineData('SCALE_FACTOR_VARIABLE_UID');
         rc = sfHash.defineDone();
         declare hash blHash(dataset: "WORK.TMP_INTERNAL_BL", hashexp: 10);
         rc = blHash.defineKey('FEED_REF_ID');
         rc = blHash.defineData('INTERNAL_BL_UID');
         rc = blHash.defineDone();
         call missing(FEED_REF_ID, SCALE_FACTOR_VARIABLE_UID, INTERNAL_BL_UID);
         call symput('DO_BL', 1);
      end;
      else do;
         id_val = id_val + 1;
      end;

      if (mod(_N_,&MOD_CNT) eq 0) then do;
         put "STATUS PROGRESS2 COUNT_INC=&MOD_CNT";
      end;

      /* Assign whatever variables are available. */
      SCALE_FACTOR_BL_UID = id_val;
      SCALE_FACTOR_YEAR = in_year;
      SCALE_FACTOR_VALUE = in_value;
      VERSION_UID = &DATA_VERSION_ID;

      /* Create a flag that will be used to determine if every item was found. */
      doOutPut = 1;

      /* Find the Scale Factor Variable. */
      FEED_REF_ID = in_sf_ref_id;
      rc = sfHash.find();
      %orvl_find_reference(rc, "&TYP_NM2 - S. Factor", WORK.TMP_LOAD_STATUS_DETAILS_SFBL);

      /* Find the Business Line. */
      FEED_REF_ID = in_bl_ref_id;
      rc = blHash.find();
      %orvl_find_reference(rc, "&TYP_NM2 - BL", WORK.TMP_LOAD_STATUS_DETAILS_SFBL);

      FEED_REF_ID = in_id;

      %orvl_cmn_test_required_value(FEED_REF_ID, "Scale_Factor_Value_BL_ID", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_SFBL);
      %orvl_cmn_test_required_value(SCALE_FACTOR_YEAR, "Scale_Factor_Year", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_SFBL);
      %orvl_cmn_test_required_value(SCALE_FACTOR_VALUE, "Scale_Factor_Value", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_SFBL);

      /* Output the item only if all of the referenced components were found. */
      if doOutPut eq 1 then do;
         output WORK.TMP_SCALE_FACTOR_BL;
      end;
   run;

   %orvl_cmn_increment_remaining(&ds_nobs,&MOD_CNT);

   %orvl_ds_exists(WORK.TMP_LOAD_STATUS_DETAILS_SFBL, "&TYP_NM2", false);
   %if &ds_nobs ne 0 %then %do;
      proc append base=WORK.LOAD_STATUS_DETAILS data=WORK.TMP_LOAD_STATUS_DETAILS_SFBL;
      run;
   %end;

%SKIPPER1:

   /* 
    * *************************************************
    * Make sure the required dataset has records. 
    * If it doesn't, orvl_ds_exists sets LOAD_STATUS to 2.
    * *************************************************
    */
   %orvl_ds_exists(&INLIB..scale_factor_value_firm, "&TYP_NM3", false);
   %if &ds_nobs eq 0 %then %do;
      %goto SKIPPER2;
   %end;

   %put STATUS PROGRESS2 TOTAL=&ds_nobs FILE="&TYP_NM3";

   /*
    * *******************************
    * Process the Scale Factor items. 
    * *******************************
    */
   /* Get the starting uid for items to insert. */
   %orvdb_seq_incrementer(&SEQ_LIB, LOAD_Scale_Factor_Firm_SEQ, dbload, INC=&ds_nobs);
   proc sql;
      select VALUE Into :THE_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;

   /* Create temporary datasets, with the new ids. */
   data WORK.TMP_SCALE_FACTOR_FIRM (KEEP= SCALE_FACTOR_FIRM_UID 
                                          FEED_REF_ID 
                                          SCALE_FACTOR_VARIABLE_UID 
                                          SCALE_FACTOR_YEAR 
                                          SCALE_FACTOR_VALUE 
                                          SCALE_FACTOR_TYPE  
                                          VERSION_UID)
        WORK.TMP_LOAD_STATUS_DETAILS_SFF (KEEP= WHO
                                                STATUS 
                                                REASON);
      set &INLIB..scale_factor_value_firm (rename=(Scale_Factor_Value_Firm_ID=in_id
                                                   Scale_Factor_ID=in_sf_ref_id
                                                   Scale_Factor_Type=in_type
                                                   Scale_Factor_Year=in_year
                                                   Scale_Factor_Value=in_value));
      length WHO $32. REASON $200.;
      length DUP_KEY $68.;

      retain id_val;

      if _N_ = 1 then do;
         id_val = symgetn('THE_ID');
         if (id_val eq .) then do;
            id_val = 1;
         end;
         declare hash sfHash(dataset: "WORK.TMP_SF_VARIABLE", hashexp: 10);
         rc = sfHash.defineKey('FEED_REF_ID');
         rc = sfHash.defineData('SCALE_FACTOR_VARIABLE_UID');
         rc = sfHash.defineDone();
         declare hash dupHash(hashexp: 10);
         rc = dupHash.defineKey('DUP_KEY');
         rc = dupHash.defineData('ITEM_EXISTS');
         rc = dupHash.defineDone();
         call missing(FEED_REF_ID, SCALE_FACTOR_VARIABLE_UID, DUP_KEY, ITEM_EXISTS);
         call symput('DO_FIRM', 1);
      end;
      else do;
         id_val = id_val + 1;
      end;

      put "STATUS PROGRESS2 COUNT_INC";
      %orvl_cmn_user_cancelled_check();
      status_cancel = symgetn('ORVAR_CANCEL');
      if status_cancel ne 0 then stop;

      /* Assign whatever variables are available. */
      SCALE_FACTOR_FIRM_UID = id_val;
      SCALE_FACTOR_TYPE = in_type;
      SCALE_FACTOR_YEAR = in_year;
      SCALE_FACTOR_VALUE = in_value;
      VERSION_UID = &DATA_VERSION_ID;
      DUP_KEY = trim(left(put(in_sf_ref_id, best.))) || '_' || trim(left(put(in_year, best.))) || '_' || trim(left(in_type));
      ITEM_EXISTS = 1;

      /* Create a flag that will be used to determine if every item was found. */
      doOutPut = 1;

      /* Find the Risk Factor Variable. */
      FEED_REF_ID = in_sf_ref_id;
      rc = sfHash.find();
      %orvl_find_reference(rc, "&TYP_NM3 - S. Factor", WORK.TMP_LOAD_STATUS_DETAILS_SFF);

      FEED_REF_ID = in_id;

      %orvl_cmn_test_required_value(FEED_REF_ID, "Scale_Factor_Value_Firm_ID", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_SFF);
      %orvl_cmn_test_required_value(SCALE_FACTOR_YEAR, "Scale_Factor_Year", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_SFF);
      %orvl_cmn_test_required_value(SCALE_FACTOR_VALUE, "Scale_Factor_Value", _N_, "&TYP_NM", WORK.TMP_LOAD_STATUS_DETAILS_SFF);

      rc = dupHash.find();
      if (rc eq 0) then do;
         %orvl_cmn_set_reason_prefix(_N_);
         temp_str = "STATUS WARNING: " || trim(left(ORVAR_REASON_OBS_PREFIX)) || " The scale factor has been previously added and will be dropped.";
         put temp_str;
         doOutPut = 0;
      end;
      else do;
         rc = dupHash.add();
         if (rc ^= 0) then do;
            %orvl_cmn_set_reason_prefix(_N_);
            temp_str = "STATUS WARNING: " || trim(left(ORVAR_REASON_OBS_PREFIX)) || " The scale factor could not be added to the check-for-dups hash.";
            put temp_str;
         end;
      end;

      /* Output the item only if all of the referenced components were found. */
      if doOutPut eq 1 then do;
         output WORK.TMP_SCALE_FACTOR_FIRM;
      end;
   run;

   %orvl_ds_exists(WORK.TMP_LOAD_STATUS_DETAILS_SFF, "&TYP_NM3", false);
   %if &ds_nobs ne 0 %then %do;
      proc append base=WORK.LOAD_STATUS_DETAILS data=WORK.TMP_LOAD_STATUS_DETAILS_SFF;
      run;
   %end;

%SKIPPER2:

   /* Append the temp datasets to the original datasets. */
   %if %TRIM(%LEFT(&LOAD_STATUS)) eq 1 %then %do;
      %orvl_cmn_proc_append(&TYP_NM, &OUTLIB..SCALE_FACTOR_VARIABLE, WORK.TMP_SF_VARIABLE);
      %orvl_cmn_user_cancelled_ck_wrap();
      %if &ORVAR_CANCEL ne 0 %then %goto DONE;
      %if %TRIM(%LEFT(&DO_BL)) eq 1 and %TRIM(%LEFT(&LOAD_STATUS)) eq 1 %then %do;
         %orvl_cmn_proc_append(&TYP_NM2, &OUTLIB..SCALE_FACTOR_BL, WORK.TMP_SCALE_FACTOR_BL);
         %orvl_cmn_user_cancelled_ck_wrap();
         %if &ORVAR_CANCEL ne 0 %then %goto DONE;
      %end;
      %if %TRIM(%LEFT(&DO_FIRM)) eq 1 and %TRIM(%LEFT(&LOAD_STATUS)) eq 1 %then %do;
         %orvl_cmn_proc_append(&TYP_NM3, &OUTLIB..SCALE_FACTOR_FIRM, WORK.TMP_SCALE_FACTOR_FIRM);
         %orvl_cmn_user_cancelled_ck_wrap();
         %if &ORVAR_CANCEL ne 0 %then %goto DONE;
      %end;
   %end;

%DONE:
   %put STATUS PROGRESS2 DONE;

   /* Handle the final signaling. */
   %orvl_cmn_status_processing;

%mend orvl_int_scale;
