/*-------------------------------------------------------------------
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * NAME:    orvr_report_setup                                             
 * PURPOSE: Setup general stuff needed by all reports. 
 *
 *          title1 is reserved for the Company Title and us set
 *          by macro %cleartitles;
 *      
 * SUPPORT: stchri
 *
 *
 * Input:
 *    fileref - fileref for HTML output.
 *    model_uid - specified what project we're dealing with.
 *
 *
 * NOTES: Assumes caller has set the following librefs:
 *        varprj, varuser, vardata
 *-------------------------------------------------------------------*/
 %macro orvr_report_setup(fileref      =,
                          model_uid    =,
                          outputtype   =,
                          style        =DEFAULT,
                          graphics     =false);
 
    option noovp nodate;
 
 	/*** Create fake data for empty reports ***/
	data _fake_;
	   length fakevar $10; label fakevar = '00'X;
	   fakevar= ' ';
	run;

   /*** fix for template issue where can't overwrite template in sasuser area ***/
       ods path(prepend) work.xxx(update);

    /*** select CURFMT and DEFFMT format ***/
    %global CURFMT;
    %let CURFMT=  NLMNY32.2;
    %global DEFFMT;
    %let DEFFMT=  NLNUM32.3;
    %global DATEFMT;
    %let DATEFMT= NLDATMAP.;
	%global DATETIMEFMT;
	%let DATETIMEFMT= NLDATMAP.;
    %global COMPANYTITLE;
    %let COMPANYTITLE= Unknown Company;
    
	data _NULL_; set vardata.VAR_PROPERTIES;
	    if trim(left(property)) eq 'ReportCurrencyFormat' and value ne '' then do;
	       call symput("CURFMT", trim(left(value)));
	    end;
	    else if trim(left(property)) eq 'ReportNumericFormat' and value ne '' then do;
	       call symput("DEFFMT", trim(left(value)));
	    end;
	    else if trim(left(property)) eq 'ReportDateFormat' and value ne '' then do;	    
	       call symput("DATEFMT", trim(left(value)));	
	    end;	
		 else if trim(left(property)) eq 'ReportDateTimeFormat' and value ne '' then do;	    
   		 call symput("DATETIMEFMT", trim(left(value)));	
		 end;	
	    else if trim(left(property)) eq 'ReportCompanyTitle' and value ne '' then do;	    
	       call symput("COMPANYTITLE", trim(left(value)));	
	    end;	
	run;
	
    /*** specifications for titles and stuff ***/
    %global titleSpec1;
	%let titleSpec1 = j=center color=CX996600 height=14pt ;
    %global titleSpec2;
	%let titleSpec2 = j=center color=black height=12pt ;
    %global titleSpec3;
	%let titleSpec3 = j=center color=gray height=10pt ;
    %global titleSpec4;
	%let titleSpec4 = j=center height=10pt;
    %global titleSpec5;
	%let titleSpec5 = j=center height=8pt;

	%macro cleartitles;
		title; title1; title2; title3; title4; title5; title6; title7; title8; 
		title1 &titleSpec1 "&COMPANYTITLE";
	%mend;
	%cleartitles;
	
	
    %global footnoteSpec1;
	%let footnoteSpec1 = j=right height=8pt color=black "Report Generated : %sysfunc(putn(%sysfunc(datetime()), &DATETIMEFMT))";
    %global reportSpec;
    %let reportSpec = nowd SPLIT="*" style(column)=[font_size=2] headline headskip CENTER;
	
	/*** Start ODS Session ***/ 
	
	%if %substr(%upcase(&outputtype),1,3) = HTM %then %do;
	   %if "&graphics" eq "true" %then %do;
	      ods tagsets.htmlpanel gpath="&_VARLIB" body=&fileref style=&style codebase="&_VARLIB";
	   %end;
	   %else %do;
	      ods &outputtype body=&fileref style=&style path="&_VARCONTENTS";
	   %end;
	%end;
	%else %do;
	   ods &outputtype body=&fileref style=&style;
	%end;
    ods listing close;

 
	/********************************************
	  Create macro variables for the following 
	    MODEL_NAME - currnt model name
	    FULL_NAME  - of model's author
	    TODAY_DATE - date report is run 
	  *******************************************/
    %global MODEL_NAME VAR_USER_UID VERSION_UID;
	data _NULL_; set varprj.MODEL;
		where MODEL_UID = &model_uid;
	    call symput("MODEL_NAME",strip(model_name));
		call symput("VAR_USER_UID",var_user_uid);
		call symput("VERSION_UID",version_uid);
	run;
	
	data var_user;
		set varuser.VAR_USER;
		where VAR_USER_UID=&VAR_USER_UID;
	run;
	
    %global FULL_NAME TODAY_DATE;
	data _NULL_;
	  merge var_user varprj.MODEL;
	  by VAR_USER_UID;
	  
	  length FULL_NAME $ 128; 
	  FULL_NAME = strip(FIRST_NAME)||" "||strip(MIDDLE_NAME)||" "||strip(LAST_NAME);
	  if anycntrl(FULL_NAME) ^=0 then Put "Control characters in Name" FULL_NAME=;
	  call symput("FULL_NAME",strip(FULL_NAME));
	
	  format dt &DATEFMT;
	  dt = datetime();
	  call symput("TODAY_DATE",left(put(dt,&DATEFMT)));
	run;
	
    proc datasets lib=work memtype=data nolist;
       delete var_user;
    run; quit;
    

	/*** create macros for MAPNAME0 and MAPNAME1 ***/
	%global MAPNAME0 MAPNAME1;
	data _NULL_;
	  set varprj.ccmapval_0; where CCVUID_0 = -999;
	  call symput("MAPNAME0",trim(CCVVALUE_0));
	run;
	data _NULL_;
	  set varprj.ccmapval_1; where CCVUID_1 = -999;
	  call symput("MAPNAME1",trim(CCVVALUE_1));
	run;
 
 %mend orvr_report_setup;